@@ -42,10 +42,14 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <ctype.h>
+#ifdef LIBSSH2DEBUG
+#include <stdio.h>
+#endif
 
 #include <assert.h>
 
 #include "transport.h"
+#include "mac.h"
 
 #define MAX_BLOCKSIZE 32    /* MUST fit biggest crypto block size we use/get */
 #define MAX_MACSIZE 20      /* MUST fit biggest MAC length we support */
@@ -54,7 +58,7 @@
 #define UNPRINTABLE_CHAR '.'
 static void
 debugdump(LIBSSH2_SESSION * session,
-          const char *desc, unsigned char *ptr, size_t size)
+          const char *desc, const unsigned char *ptr, size_t size)
 {
     size_t i;
     size_t c;
@@ -74,7 +78,7 @@ debugdump(LIBSSH2_SESSION * session,
         (session->tracehandler)(session, session->tracehandler_context,
                                 buffer, used);
     else
-        write(2 /* stderr */, buffer, used);
+        fprintf(stderr, "%s", buffer);
 
     for(i = 0; i < size; i += width) {
 
@@ -110,7 +114,7 @@ debugdump(LIBSSH2_SESSION * session,
             (session->tracehandler)(session, session->tracehandler_context,
                                     buffer, used);
         else
-            write(2, buffer, used);
+            fprintf(stderr, "%s", buffer);
     }
 }
 #else
@@ -138,7 +142,7 @@ decrypt(LIBSSH2_SESSION * session, unsigned char *source,
         if (session->remote.crypt->crypt(session, source,
                                          &session->remote.crypt_abstract)) {
             LIBSSH2_FREE(session, p->payload);
-            return LIBSSH2_ERROR_SOCKET_NONE;
+            return LIBSSH2_ERROR_DECRYPT;
         }
 
         /* if the crypt() function would write to a given address it
@@ -196,49 +200,28 @@ fullpacket(LIBSSH2_SESSION * session, int encrypted /* 1 or 0 */ )
 
         /* Check for and deal with decompression */
         if (session->remote.comp &&
-            strcmp(session->remote.comp->name, "none")) {
+            session->remote.comp->compress &&
+            session->remote.comp_abstract) {
+            /*
+             * The buffer for the decompression (remote.comp_abstract) is
+             * initialised in time when it is needed so as long it is NULL we
+             * cannot decompress.
+             */
+
             unsigned char *data;
             size_t data_len;
-            int free_payload = 1;
+            rc = session->remote.comp->decomp(session,
+                                              &data, &data_len,
+                                              LIBSSH2_PACKET_MAXDECOMP,
+                                              p->payload,
+                                              session->fullpacket_payload_len,
+                                              &session->remote.comp_abstract);
+            LIBSSH2_FREE(session, p->payload);
+            if(rc)
+                return rc;
 
-            if (session->remote.comp->comp(session, 0,
-                                           &data, &data_len,
-                                           LIBSSH2_PACKET_MAXDECOMP,
-                                           &free_payload,
-                                           p->payload,
-                                           session->fullpacket_payload_len,
-                                           &session->remote.comp_abstract)) {
-                LIBSSH2_FREE(session, p->payload);
-                return LIBSSH2_ERROR_SOCKET_NONE;
-            }
-
-            if (free_payload) {
-                LIBSSH2_FREE(session, p->payload);
-                p->payload = data;
-                session->fullpacket_payload_len = data_len;
-            } else {
-                if (data == p->payload) {
-                    /* It's not to be freed, because the
-                     * compression layer reused payload, So let's
-                     * do the same!
-                     */
-                    session->fullpacket_payload_len = data_len;
-                } else {
-                    /* No comp_method actually lets this happen,
-                     * but let's prepare for the future */
-
-                    LIBSSH2_FREE(session, p->payload);
-
-                    /* We need a freeable struct otherwise the
-                     * brigade won't know what to do with it */
-                    p->payload = LIBSSH2_ALLOC(session, data_len);
-                    if (!p->payload)
-                        return LIBSSH2_ERROR_ALLOC;
-
-                    memcpy(p->payload, data, data_len);
-                    session->fullpacket_payload_len = data_len;
-                }
-            }
+            p->payload = data;
+            session->fullpacket_payload_len = data_len;
         }
 
         session->fullpacket_packet_type = p->payload[0];
@@ -280,7 +263,7 @@ fullpacket(LIBSSH2_SESSION * session, int encrypted /* 1 or 0 */ )
  */
 int _libssh2_transport_read(LIBSSH2_SESSION * session)
 {
-    int rc = LIBSSH2_ERROR_SOCKET_NONE;
+    int rc;
     struct transportpacket *p = &session->packet;
     int remainbuf;
     int remainpack;
@@ -289,6 +272,7 @@ int _libssh2_transport_read(LIBSSH2_SESSION * session)
     unsigned char block[MAX_BLOCKSIZE];
     int blocksize;
     int encrypted = 1;
+    size_t total_num;
 
     /* default clear the bit */
     session->socket_block_directions &= ~LIBSSH2_SESSION_BLOCK_INBOUND;
@@ -376,25 +360,22 @@ int _libssh2_transport_read(LIBSSH2_SESSION * session)
                 _libssh2_recv(session->socket_fd, &p->buf[remainbuf],
                               PACKETBUFSIZE - remainbuf,
                               LIBSSH2_SOCKET_RECV_FLAGS(session));
-            if (nread < 0)
-                _libssh2_debug(session, LIBSSH2_TRACE_SOCKET,
-                               "Error recving %d bytes to %p+%d: %d",
-                               PACKETBUFSIZE - remainbuf, p->buf, remainbuf,
-                               errno);
-            else
-                _libssh2_debug(session, LIBSSH2_TRACE_SOCKET,
-                               "Recved %d/%d bytes to %p+%d", nread,
-                               PACKETBUFSIZE - remainbuf, p->buf, remainbuf);
             if (nread <= 0) {
                 /* check if this is due to EAGAIN and return the special
                    return code if so, error out normally otherwise */
-                if ((nread < 0) && (errno == EAGAIN)) {
+                if ((nread < 0) && (nread == -EAGAIN)) {
                     session->socket_block_directions |=
                         LIBSSH2_SESSION_BLOCK_INBOUND;
                     return LIBSSH2_ERROR_EAGAIN;
                 }
-                return LIBSSH2_ERROR_SOCKET_NONE;
+                _libssh2_debug(session, LIBSSH2_TRACE_SOCKET,
+                               "Error recving %d bytes (got %d)",
+                               PACKETBUFSIZE - remainbuf, -nread);
+                return LIBSSH2_ERROR_SOCKET_RECV;
             }
+            _libssh2_debug(session, LIBSSH2_TRACE_SOCKET,
+                           "Recved %d/%d bytes to %p+%d", nread,
+                           PACKETBUFSIZE - remainbuf, p->buf, remainbuf);
 
             debugdump(session, "libssh2_transport_read() raw",
                       &p->buf[remainbuf], nread);
@@ -445,13 +426,13 @@ int _libssh2_transport_read(LIBSSH2_SESSION * session)
              */
             p->packet_length = _libssh2_ntohu32(block);
             if (p->packet_length < 1)
-                return LIBSSH2_ERROR_SOCKET_NONE;
+                return LIBSSH2_ERROR_DECRYPT;
 
             p->padding_length = block[4];
 
             /* total_num is the number of bytes following the initial
                (5 bytes) packet length and padding length fields */
-            p->total_num =
+            total_num =
                 p->packet_length - 1 +
                 (encrypted ? session->remote.mac->mac_len : 0);
 
@@ -463,16 +444,17 @@ int _libssh2_transport_read(LIBSSH2_SESSION * session)
              * or less (including length, padding length, payload,
              * padding, and MAC.)."
              */
-            if (p->total_num > LIBSSH2_PACKET_MAXPAYLOAD) {
+            if (total_num > LIBSSH2_PACKET_MAXPAYLOAD) {
                 return LIBSSH2_ERROR_OUT_OF_BOUNDARY;
             }
 
             /* Get a packet handle put data into. We get one to
                hold all data, including padding and MAC. */
-            p->payload = LIBSSH2_ALLOC(session, p->total_num);
+            p->payload = LIBSSH2_ALLOC(session, total_num);
             if (!p->payload) {
                 return LIBSSH2_ERROR_ALLOC;
             }
+            p->total_num = total_num;
             /* init write pointer to start of payload buffer */
             p->wptr = p->payload;
 
@@ -597,32 +579,18 @@ int _libssh2_transport_read(LIBSSH2_SESSION * session)
         }
     } while (1);                /* loop */
 
-    return LIBSSH2_ERROR_SOCKET_NONE;         /* we never reach this point */
-}
-
-/*
- * _libssh2_transport_drain() empties the outgoing send buffer if there
- * is any.
- */
-void _libssh2_transport_drain(LIBSSH2_SESSION * session)
-{
-    struct transportpacket *p = &session->packet;
-    if(p->outbuf) {
-        LIBSSH2_FREE(session, p->outbuf);
-        p->outbuf = NULL;
-        p->ototal_num = 0;
-    }
+    return LIBSSH2_ERROR_SOCKET_RECV; /* we never reach this point */
 }
 
 static int
-send_existing(LIBSSH2_SESSION * session, unsigned char *data,
-              size_t data_len, ssize_t * ret)
+send_existing(LIBSSH2_SESSION *session, const unsigned char *data,
+              size_t data_len, ssize_t *ret)
 {
     ssize_t rc;
     ssize_t length;
     struct transportpacket *p = &session->packet;
 
-    if (!p->outbuf) {
+    if (!p->olen) {
         *ret = 0;
         return LIBSSH2_ERROR_NONE;
     }
@@ -646,29 +614,31 @@ send_existing(LIBSSH2_SESSION * session, unsigned char *data,
                        LIBSSH2_SOCKET_SEND_FLAGS(session));
     if (rc < 0)
         _libssh2_debug(session, LIBSSH2_TRACE_SOCKET,
-                       "Error sending %d bytes: %d", length, errno);
-    else
+                       "Error sending %d bytes: %d", length, -rc);
+    else {
         _libssh2_debug(session, LIBSSH2_TRACE_SOCKET,
                        "Sent %d/%d bytes at %p+%d", rc, length, p->outbuf,
                        p->osent);
-
-    if(rc > 0) {
         debugdump(session, "libssh2_transport_write send()",
                   &p->outbuf[p->osent], rc);
     }
 
     if (rc == length) {
         /* the remainder of the package was sent */
-        LIBSSH2_FREE(session, p->outbuf);
-        p->outbuf = NULL;
         p->ototal_num = 0;
+        p->olen = 0;
+        /* we leave *ret set so that the parent returns as we MUST return back
+           a send success now, so that we don't risk sending EAGAIN later
+           which then would confuse the parent function */
+        return LIBSSH2_ERROR_NONE;
+
     }
     else if (rc < 0) {
         /* nothing was sent */
-        if (errno != EAGAIN) {
+        if (rc != -EAGAIN)
             /* send failure! */
-            return LIBSSH2_ERROR_SOCKET_NONE;
-        }
+            return LIBSSH2_ERROR_SOCKET_SEND;
+
         session->socket_block_directions |= LIBSSH2_SESSION_BLOCK_OUTBOUND;
         return LIBSSH2_ERROR_EAGAIN;
     }
@@ -679,48 +649,50 @@ send_existing(LIBSSH2_SESSION * session, unsigned char *data,
 }
 
 /*
- * libssh2_transport_write
+ * libssh2_transport_send
  *
  * Send a packet, encrypting it and adding a MAC code if necessary
  * Returns 0 on success, non-zero on failure.
  *
- * Returns LIBSSH2_ERROR_EAGAIN if it would block - and if it does so, you should
- * call this function again as soon as it is likely that more data can be
- * sent, and this function should then be called with the same argument set
- * (same data pointer and same data_len) until zero or failure is returned.
+ * The data is provided as _two_ data areas that are combined by this
+ * function.  The 'data' part is sent immediately before 'data2'. 'data2' may
+ * be set to NULL to only use a single part.
  *
- * NOTE: this function does not verify that 'data_len' is less than ~35000
- * which is what all implementations should support at least as packet size.
- * (RFC4253 section 6.1)
+ * Returns LIBSSH2_ERROR_EAGAIN if it would block or if the whole packet was
+ * not sent yet. If it does so, the caller should call this function again as
+ * soon as it is likely that more data can be sent, and this function MUST
+ * then be called with the same argument set (same data pointer and same
+ * data_len) until ERROR_NONE or failure is returned.
  *
- * This function DOES not call _libssh2_error() on any errors.
+ * This function DOES NOT call _libssh2_error() on any errors.
  */
-int
-_libssh2_transport_write(LIBSSH2_SESSION * session, unsigned char *data,
-                         size_t data_len)
+int _libssh2_transport_send(LIBSSH2_SESSION *session,
+                            const unsigned char *data, size_t data_len,
+                            const unsigned char *data2, size_t data2_len)
 {
     int blocksize =
-        (session->state & LIBSSH2_STATE_NEWKEYS) ? session->local.crypt->
-        blocksize : 8;
+        (session->state & LIBSSH2_STATE_NEWKEYS) ?
+        session->local.crypt->blocksize : 8;
     int padding_length;
-    int packet_length;
+    size_t packet_length;
     int total_length;
-    int free_data = 0;
 #ifdef RANDOM_PADDING
     int rand_max;
     int seed = data[0];         /* FIXME: make this random */
 #endif
     struct transportpacket *p = &session->packet;
     int encrypted;
-    int i;
     ssize_t ret;
     int rc;
-    unsigned char *orgdata = data;
+    const unsigned char *orgdata = data;
     size_t orgdata_len = data_len;
 
     debugdump(session, "libssh2_transport_write plain", data, data_len);
+    if(data2)
+        debugdump(session, "libssh2_transport_write plain2", data2, data2_len);
 
-    /* FIRST, check if we have a pending write to complete */
+    /* FIRST, check if we have a pending write to complete. send_existing
+       only sanity-check data and data_len and not data2 and data2_len!! */
     rc = send_existing(session, data, data_len, &ret);
     if (rc)
         return rc;
@@ -728,19 +700,56 @@ _libssh2_transport_write(LIBSSH2_SESSION * session, unsigned char *data,
     session->socket_block_directions &= ~LIBSSH2_SESSION_BLOCK_OUTBOUND;
 
     if (ret)
+        /* set by send_existing if data was sent */
         return rc;
 
     encrypted = (session->state & LIBSSH2_STATE_NEWKEYS) ? 1 : 0;
 
-    /* check if we should compress */
-    if (encrypted && strcmp(session->local.comp->name, "none")) {
-        if (session->local.comp->comp(session, 1, &data, &data_len,
-                                      LIBSSH2_PACKET_MAXCOMP,
-                                      &free_data, data, data_len,
-                                      &session->local.comp_abstract)) {
-            return LIBSSH2_ERROR_COMPRESS;     /* compression failure */
+    if (encrypted && session->local.comp->compress) {
+        /* the idea here is that these function must fail if the output gets
+           larger than what fits in the assigned buffer so thus they don't
+           check the input size as we don't know how much it compresses */
+        size_t dest_len = MAX_SSH_PACKET_LEN-5-256;
+        size_t dest2_len = dest_len;
+
+        /* compress directly to the target buffer */
+        rc = session->local.comp->comp(session,
+                                       &p->outbuf[5], &dest_len,
+                                       data, data_len,
+                                       &session->local.comp_abstract);
+        if(rc)
+            return rc;     /* compression failure */
+
+        if(data2 && data2_len) {
+            /* compress directly to the target buffer right after where the
+               previous call put data */
+            dest2_len -= dest_len;
+
+            rc = session->local.comp->comp(session,
+                                           &p->outbuf[5+dest_len], &dest2_len,
+                                           data2, data2_len,
+                                           &session->local.comp_abstract);
         }
+        else
+            dest2_len = 0;
+        if(rc)
+            return rc;     /* compression failure */
+
+        data_len = dest_len + dest2_len; /* use the combined length */
     }
+    else {
+        if((data_len + data2_len) >= (MAX_SSH_PACKET_LEN-0x100))
+            /* too large packet, return error for this until we make this
+               function split it up and send multiple SSH packets */
+            return LIBSSH2_ERROR_INVAL;
+
+        /* copy the payload data */
+        memcpy(&p->outbuf[5], data, data_len);
+        if(data2 && data2_len)
+            memcpy(&p->outbuf[5+data_len], data2, data2_len);
+        data_len += data2_len; /* use the combined length */
+    }
+
 
     /* RFC4253 says: Note that the length of the concatenation of
        'packet_length', 'padding_length', 'payload', and 'random padding'
@@ -781,28 +790,18 @@ _libssh2_transport_write(LIBSSH2_SESSION * session, unsigned char *data,
     total_length =
         packet_length + (encrypted ? session->local.mac->mac_len : 0);
 
-    /* allocate memory to store the outgoing packet in, in case we can't
-       send the whole one and thus need to keep it after this function
-       returns. */
-    p->outbuf = LIBSSH2_ALLOC(session, total_length);
-    if (!p->outbuf) {
-        return LIBSSH2_ERROR_ALLOC;
-    }
-
     /* store packet_length, which is the size of the whole packet except
        the MAC and the packet_length field itself */
     _libssh2_htonu32(p->outbuf, packet_length - 4);
     /* store padding_length */
     p->outbuf[4] = padding_length;
-    /* copy the payload data */
-    memcpy(p->outbuf + 5, data, data_len);
+
     /* fill the padding area with random junk */
     _libssh2_random(p->outbuf + 5 + data_len, padding_length);
-    if (free_data) {
-        LIBSSH2_FREE(session, data);
-    }
 
     if (encrypted) {
+        size_t i;
+
         /* Calculate MAC hash. Put the output at index packet_length,
            since that size includes the whole packet. The MAC is
            calculated on the entire unencrypted packet, including all
@@ -818,7 +817,7 @@ _libssh2_transport_write(LIBSSH2_SESSION * session, unsigned char *data,
             unsigned char *ptr = &p->outbuf[i];
             if (session->local.crypt->crypt(session, ptr,
                                             &session->local.crypt_abstract))
-                return LIBSSH2_ERROR_SOCKET_NONE;     /* encryption failure */
+                return LIBSSH2_ERROR_ENCRYPT;     /* encryption failure */
         }
     }
 
@@ -828,32 +827,29 @@ _libssh2_transport_write(LIBSSH2_SESSION * session, unsigned char *data,
                         LIBSSH2_SOCKET_SEND_FLAGS(session));
     if (ret < 0)
         _libssh2_debug(session, LIBSSH2_TRACE_SOCKET,
-                       "Error sending %d bytes: %d", total_length, errno);
-    else
+                       "Error sending %d bytes: %d", total_length, -ret);
+    else {
         _libssh2_debug(session, LIBSSH2_TRACE_SOCKET, "Sent %d/%d bytes at %p",
                        ret, total_length, p->outbuf);
-
-    if (ret != -1) {
         debugdump(session, "libssh2_transport_write send()", p->outbuf, ret);
     }
+
     if (ret != total_length) {
-        if ((ret > 0) || ((ret == -1) && (errno == EAGAIN))) {
+        if (ret >= 0 || ret == -EAGAIN) {
             /* the whole packet could not be sent, save the rest */
             session->socket_block_directions |= LIBSSH2_SESSION_BLOCK_OUTBOUND;
             p->odata = orgdata;
             p->olen = orgdata_len;
-            p->osent = (ret == -1) ? 0 : ret;
+            p->osent = ret <= 0 ? 0 : ret;
             p->ototal_num = total_length;
             return LIBSSH2_ERROR_EAGAIN;
         }
-        return LIBSSH2_ERROR_SOCKET_NONE;
+        return LIBSSH2_ERROR_SOCKET_SEND;
     }
 
     /* the whole thing got sent away */
     p->odata = NULL;
     p->olen = 0;
-    LIBSSH2_FREE(session, p->outbuf);
-    p->outbuf = NULL;
 
     return LIBSSH2_ERROR_NONE;         /* all is good */
 }
